@@ -114,7 +114,7 @@ class DeReference(object):
                             doc = get_document(ref["_cls"])._from_son(ref)
                         elif doc_type is None:
                             doc = get_document(
-                                ''.join(x.capitalize() 
+                                ''.join(x.capitalize()
                                         for x in col.split('_')))._from_son(ref)
                         else:
                             doc = doc_type._from_son(ref)
@@ -166,7 +166,7 @@ class DeReference(object):
             else:
                 data[k] = v
 
-            if k in self.object_map:
+            if k in self.object_map and not is_list:
                 data[k] = self.object_map[k]
             elif hasattr(v, '_fields'):
                 for field_name, field in v._fields.iteritems():
